package org.processmining.streamalphaminer.algorithms;

import org.deckfour.xes.classification.XEventClass;
import org.processmining.acceptingpetrinet.models.AcceptingPetriNet;
import org.processmining.alphaminer.abstractions.AlphaClassicAbstraction;
import org.processmining.alphaminer.abstractions.impl.AlphaClassicAbstractionImpl;
import org.processmining.alphaminer.algorithms.AlphaClassicMinerImpl;
import org.processmining.alphaminer.parameters.AlphaMinerParameters;
import org.processmining.alphaminer.parameters.AlphaVersion;
import org.processmining.eventstream.readers.abstr.AbstractXSEventReader;
import org.processmining.eventstream.readers.acceptingpetrinet.AcceptingPetriNetXSReaderResultVisualizer;
import org.processmining.eventstream.readers.acceptingpetrinet.XSEventStreamToAcceptingPetriNetReader;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.logabstractions.factories.StartEndActivityFactory;
import org.processmining.logabstractions.models.DirectlyFollowsAbstraction;
import org.processmining.logabstractions.models.EndActivityAbstraction;
import org.processmining.logabstractions.models.StartActivityAbstraction;
import org.processmining.stream.reader.views.XSReaderSequenceOfResultsView;
import org.processmining.streamabstractrepresentation.algorithms.UncoupledXEventClassDFAReaderImpl;
import org.processmining.streamabstractrepresentation.parameters.UnCoupledAbstractionSchemeParametersImpl;

public class UncoupledStreamAlphaAPNXSEventReaderImpl
		extends AbstractXSEventReader<AcceptingPetriNet, AcceptingPetriNet, UnCoupledAbstractionSchemeParametersImpl>
		implements XSEventStreamToAcceptingPetriNetReader {

	private final PluginContext context;
	private final UncoupledXEventClassDFAReaderImpl<Object, UnCoupledAbstractionSchemeParametersImpl> dfaDelegate;

	public UncoupledStreamAlphaAPNXSEventReaderImpl(PluginContext context,
			final UnCoupledAbstractionSchemeParametersImpl params) {
		super("Stream Alpha Miner (Accepting Petri Net, CxA: "
				+ params.getCaseAdministrationDataStructureType().getName() + "-"
				+ params.getCaseAdministrationDataStructureParameters().toString() + "; AxA: "
				+ params.getInternalRepresentationDataStructureType().getName() + "-"
				+ params.getInternalRepresentationDataStructureType().toString() + ")",
				new XSReaderSequenceOfResultsView<>("Visualizer of: Stream Alpha Miner (Accepting Petri Net)", null,
						params.getRefreshRate(), AcceptingPetriNetXSReaderResultVisualizer.instance(context)),
				params);
		this.context = context;
		dfaDelegate = new UncoupledXEventClassDFAReaderImpl<Object, UnCoupledAbstractionSchemeParametersImpl>(
				"DFA Reader (Stream Alpha Miner)", params);
		((XSReaderSequenceOfResultsView<AcceptingPetriNet>) getVisualization()).setReader(this);
	}

	protected AcceptingPetriNet computeCurrentResult() {
		DirectlyFollowsAbstraction<XEventClass> dfa = dfaDelegate.getCurrentResult();
		AlphaClassicAbstraction<XEventClass> abstraction = new AlphaClassicAbstractionImpl<>(dfa.getEventClasses(), dfa,
				constructStartActivityAbstraction(dfa), constructEndActivityAbstraction(dfa));
		AlphaMinerParameters params = new AlphaMinerParameters();
		params.setVersion(AlphaVersion.CLASSIC);
		AlphaClassicMinerImpl<XEventClass, AlphaClassicAbstraction<XEventClass>, AlphaMinerParameters> miner = new AlphaClassicMinerImpl<>(
				params, abstraction, context);
		return miner.runAccPN();
	}

	private StartActivityAbstraction<XEventClass> constructStartActivityAbstraction(
			DirectlyFollowsAbstraction<XEventClass> dfa) {
		double[] startActivities = new double[dfa.getEventClasses().length];
		for (int i = 0; i < startActivities.length; i++) {
			if (dfa.getAllGeqForColumn(i).isEmpty()) {
				startActivities[i] = StartEndActivityFactory.DEFAULT_THRESHOLD_BOOLEAN;
			}
		}
		return StartEndActivityFactory.constructStartActivityAbstraction(dfa.getEventClasses(), startActivities,
				StartEndActivityFactory.DEFAULT_THRESHOLD_BOOLEAN);
	}

	private EndActivityAbstraction<XEventClass> constructEndActivityAbstraction(
			DirectlyFollowsAbstraction<XEventClass> dfa) {
		double[] endActivities = new double[dfa.getEventClasses().length];
		for (int i = 0; i < endActivities.length; i++) {
			if (dfa.getAllGeqForRow(i).isEmpty()) {
				endActivities[i] = StartEndActivityFactory.DEFAULT_THRESHOLD_BOOLEAN;
			}
		}
		return StartEndActivityFactory.constructEndActivityAbstraction(dfa.getEventClasses(), endActivities,
				StartEndActivityFactory.DEFAULT_THRESHOLD_BOOLEAN);
	}

	public void processNewXSEvent(String caseId, XEventClass activity) {
		dfaDelegate.processNewXSEvent(caseId, activity);
	}
}
